% Example 5.5 Wind Circulation in Thermaikos Gulf
% ho = Water depth [m];
% u = Velocity in the x-direction [m/s];
% v = Velocity in the y-direction [m/s];
% wx = Wind velocity along the x-axis [m/s];
% wy = Wind velocity along the y-axis [m/s];
% dif = Lower limit of the eddy viscosity;
% cf = Coriolis coefficient [1/s];
% fs = Water surface friction coefficient;
% fb = Bed friction coefficient;
% sp = Smagorinsky parameter for horizontal eddy coefficent [dimensionless];
% Dd = Spatial step in both x and y directions [m];
% Dt = Time step [s];
% nx = Number of computational steps in the x-direction;
% ny = Number of computational steps in the y-direction;
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
g=9.81;
ho=0;
wx=7;
wy=-7;
dif=1;
cf=0.0001;
fs=0.000003;
fb=0.01;
sp=0.3;
Dd=2000;
Dt=30;
nx=22;
ny=23;
nt=21600;
load ThermD.txt
df=ThermD;
% Import h=fm(i,j,df): Subroutine that defines depths:
% Initial conditions;
for i=1:nx
    for j=1:ny
        u(i,j)=0;
        un(i,j)=0;
        v(i,j)=0;
        vn(i,j)=0;
        z(i,j)=0;
    end
end
% Main program;
for k=1:nt
    % Estimation of the eddy viscosity;
    for i=2:nx-1
        for j=2:ny-1
            t1=(u(i+1,j)-u(i,j))/Dd-(v(i,j+1)-v(i,j))/Dd;
            t2=(u(i,j+1)+u(i+1,j+1)-u(i,j-1)-u(i+1,j-1))/2/Dd+(v(i+1,j+1)+v(i+1,j)-v(i-1,j+1)-v(i-1,j))/4/Dd;
            ev(i,j)=sp*Dd^2*sqrt(t1^2+t2^2);
            if ev(i,j) < 1;
                ev(i,j)=1;
            end
        end
    end
    % Solution of the continuity equation;
    for i=2:nx-2
        for j=2:ny-2
            h1=fm(i,j,df);
            h2=fm(i-1,j,df);
            h3=fm(i+1,j,df);
            h4=fm(i,j+1,df);
            h5=fm(i,j-1,df);
            if h1>0.1
                hl=(h1+h2)/2;
                hr=(h1+h3)/2;
                ho=(h4+h1)/2;
                hu=(h1+h5)/2;
                z(i,j)=z(i,j)-Dt/Dd*(u(i+1,j)*hr-u(i,j)*hl+v(i,j+1)*ho-v(i,j)*hu);
            end
        end
    end
    % Solution x-axis equilibrium equation;
    for i=3:nx-2
        for j=2:ny-2
            evd=(ev(i,j)+ev(i-1,j))/2;
            h1=fm(i,j,df);
            h2=fm(i-1,j,df);
            if h1>0 && h2>0
                hm=(h1+h2)/2;
                vm=(v(i,j)+v(i,j+1)+v(i-1,j)+v(i-1,j+1))/4;
                surfx=fs*wx*sqrt(wx^2+wy^2)/hm;
                adv=-u(i,j)*(u(i+1,j)-u(i-1,j))/2/Dd-vm*(u(i,j+1)-u(i,j-1))/2/Dd;
                cor=vm*cf;
                frict=-fb*sqrt(u(i,j)^2+vm^2)*u(i,j)/hm;
                pres=-g*(z(i,j)-z(i-1,j))/Dd;
                diff=evd*(-4*u(i,j)+u(i+1,j)+u(i-1,j)+u(i,j+1)+u(i,j-1))/Dd^2;
                un(i,j)=u(i,j)+Dt*(surfx+cor+pres+frict+diff+adv);
            end
        end
    end
    % Boundary conditions for velocity u;
    for j=2:ny-2
        h8=fm(2,j,df);
        if h8>0.1
            un(2,j)=-z(2,j)*sqrt(g/h8);
        end
        h9=fm(nx-2,j,df);
        if h9>0.1
            un(nx-1,j)=z(nx-2,j)*sqrt(g/h9);
        end
    end
    for i=2:nx-2
        un(i,1)=un(i,2);
        un(i,ny-1)=un(i,ny-2);
    end
    % Solution of the y-axis equilibrium condition;
    for i=2:nx-2
        for j=3:ny-1
            edd=(ev(i,j)+ev(i,j-1))/2;
            h1=fm(i,j,df);
            h5=fm(i,j-1,df);
            if h1>0 && h5>0
                hm=(h1+h5)/2;
                um=(u(i,j)+u(i+1,j)+u(i,j-1)+u(i+1,j-1))/4;
                surfy=fs*wy*sqrt(wx^2+wy^2)/hm;
                adv=-v(i,j)*(v(i,j+1)-v(i,j-1))/2/Dd-um*(v(i+1,j)-v(i-1,j))/2/Dd;
                cor=-um*cf;
                frict=-fb*sqrt(v(i,j)^2+um^2)*v(i,j)/hm;
                pres=-g*(z(i,j)-z(i,j-1))/Dd;
                diff=evd*(-4*v(i,j)+v(i,j-1)+v(i,j+1)+v(i-1,j)+v(i+1,j))/Dd^2;
                vn(i,j)=v(i,j)+Dt*(surfy+cor+pres+frict+diff+adv);
            end
        end
    end
    % Boundary conditions for velocity v:
    for i=2:nx-2
        h6=fm(i,2,df);
        if h6>0.1
            vn(i,2)=-z(i,2)*sqrt(g/h6);
        end
        h7=fm(i,ny-2,df);
        if h7>0.1
            vn(i,ny-1)=z(i,ny-2)*sqrt(g/h7);
        end
    end
    for j=2:ny-2
        vn(1,j)=vn(2,j);
        vn(nx-1,j)=vn(nx-2,j);
    end
    % Updating the velocities u and v;
    for i=1:nx
        for j=1:ny
            u(i,j)=un(i,j);
            v(i,j)=vn(i,j);
        end
    end
    % Calculation of the kinetic energy;
    ke=0;
    for i=1:nx
        for j=1:ny
            ke=ke+u(i,j)^2+v(i,j)^2;
        end
    end
    KinE(k)=ke;
    index=k
end
[X,Y]=meshgrid(1:1:nx,1:1:ny);
[m n]=size(Y);
for a=1:m
    for b=1:n
        Z(a,b)=-fm(b,a,df);
    end
end
surf(X,Y,Z)
figure
plot(1:k,KinE,'Color','k','Linewidth',1.5);
xlabel('Number of time steps')
ylabel('Total kinetic energy (u^2+v^2 on all grid points)')
[i,j]=meshgrid(1:1:nx,1:1:ny);
df1=df(:,3);
df2=reshape(df1,22,23);
df2=df2';
An=ones(size(df2));
idx=find(df2==0);
An(idx)=0;
figure,pcolor(An); hold on;
colormap(bone)
shading flat
LL=quiver(i,j,u',v','Color','k');
% set(LL,'linewidth',1.5);

