% Example 5.6 Wind Induced Upwelling in Stratified Water
% w = Wind velocity [m/s];
% fs = Friction coefficient at water surface;
% fi = Friction coefficient at fluid interface;
% fb = Friction coefficient at bed;
% rdd = Relative density difference of the fluids;
% ev = Eddy viscosity coefficient [m^2/s];
% huo = Initial upper fluid layer depth [m];
% hbo = Initial lower fluid layer depth [m];
% Dx = Spatial step [m];
% Dt = Time step [s];
% nx = Number of spatial computational steps;
% nt = Number of time computational steps;
clc; clear all; close all;
% Input data;
g=9.81;
w=10;
fs=0.000003;
fi=0.001;
fb=0.05;
rdd=0.005;
ev=20;
huo=10;
hbo=10;
Dx=200;
Dt=10;
nx=50;
nt=103680;
% Initial conditions;
for k=1:nx-1
    hu(k)=huo;
    hb(k)=hbo;
    hun(k)=huo;
    hbn(k)=hbo;
end
for k1=1:nx
    zb(k1)=0;
    uu(k1)=0;
    ub(k1)=0;
    uun(k1)=0;
    ubn(k1)=0;
end
% Main program;
for i=1:nt
    % Continuity equation for the upper fluid layer;
    for j=2:nx-2
        qul(j)=uu(j)*(hu(j)+hu(j-1))/2;
        qur(j)=uu(j+1)*(hu(j)+hu(j+1))/2;
        hun(j)=hu(j)-(Dt/Dx)*(qur(j)-qul(j));
    end
    % Continuity equation for the lower fluid layer;
    for j=2:nx-2
        qbl(j)=ub(j)*(hb(j)+hb(j-1))/2;
        qbr(j)=ub(j+1)*(hb(j)+hb(j+1))/2;
        hbn(j)=hb(j)-(Dt/Dx)*(qbr(j)-qbl(j));
    end
    % Updating of layer depth values;
    for j=2:nx-2
        hu(j)=hun(j);
        hb(j)=hbn(j);
    end
    % Force equilibrium equation for the upper layer;
    for j=3:nx-2
        hum(j)=(hu(j)+hu(j-1))/2;
        ts(j)=fs*w*abs(w)/hum(j);
        ti(j)=fi*(uu(j)-ub(j))*abs(uu(j)-ub(j))/hum(j);
        ed(j)=ev*(uu(j-1)-2*uu(j)+uu(j+1))/Dx^2;
        uun(j)=uu(j)-Dt*g*(hu(j)+hb(j)+zb(j)-hu(j-1)-hb(j-1)-zb(j-1))/Dx+Dt*(ed(j)+ts(j)-ti(j));
    end
    % Force equilibrium equation for the lower layer;
    for j=3:nx-2
        hbm(j)=(hb(j)+hb(j-1))/2;
        ti(j)=fi*(uu(j)-ub(j))*abs(uu(j)-ub(j))/hbm(j);
        tb(j)=fb*ub(j)*abs(ub(j))/hbm(j);
        ed(j)=ev*(ub(j-1)-2*ub(j)+ub(j+1))/Dx^2;
        ubn(j)=ub(j)-Dt*g*((hu(j)+hb(j)+zb(j)-hu(j-1)-hb(j-1)-zb(j-1))/Dx-rdd*(hu(j)-hu(j-1))/Dx)+Dt*(ed(j)+ti(j)-tb(j));
    end
    % Updating of fluid velocity values;
    for j=3:nx-2
        uu(j)=uun(j);
        ub(j)=ubn(j);
        Hbfinal(j)=hbn(j);
        Htotal(j)=hun(j)+hbn(j);
    end
    Eu=0;
    El=0;
    for j=3:nx-2
        Eu=Eu+(hu(j)+hu(j-1))/2*uu(j)^2;
        El=El+(hb(j)+hb(j-1))/2*ub(j)^2;
    end
    Hupper(i)=hu(nx-2);
    Hlower(i)=hb(nx-2);
end
plot(1:j,Hbfinal,'r','Linewidth',1.5)
hold on
plot(1:j,Htotal,'b','Linewidth',1.5)
axis([3,48,0,21])
xlabel('Distance x 200 [m]'), ylabel('Water surface and interface elevation [m]')
text(10,18.9,'Water free-surface')
text(20,11.5,'Interface at the end of simulation (t = nt*Dt)')
text(25,18,'(Wind direction is from left to right)')
figure, plot(1:i,Hupper,'b','Linewidth',1.5)
hold on
plot(1:i,Hlower,'r','Linewidth',1.5)
axis([0,100000,6.5,13.5])
xlabel('Time x 10 [s]'), ylabel('Surface oscillation at right boundary [m]')
text(40000,12,'Upper layer')
text(40000,8,'Lower layer')
